import pandas as pd
import pandas as pd
from sklearn.compose import ColumnTransformer
from sklearn.ensemble import RandomForestRegressor, GradientBoostingRegressor
from sklearn.impute import SimpleImputer
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LinearRegression, Lasso
from sklearn.metrics import mean_squared_error, r2_score
from sklearn.neural_network import MLPRegressor
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler, OneHotEncoder
from sklearn.svm import SVR


def clean_dataset(dataset):
    # Remoção Colunas
    collumns_to_remove = ["Unnamed: 0", 'D1', 'E1', 'E2', 'E3', 'E4', 'E5', 'E6', 'E7', 'E8',
                          'E9', 'E10', 'C2', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9',
                          'C10', 'C11']
    dataset = dataset.drop(collumns_to_remove, axis=1)

    # ----------------------Procedimento DatasetPreparation ---------------------------------------------
    jogos_benfica = pd.read_csv("D:\ISCTE\Internet das Coisas\D\Jogos Benfica.csv", sep=';')
    grelha_lisboa = pd.read_csv("D:\ISCTE\Internet das Coisas\D\VODAFONE_QUADRICULAS\VODAFONE_QUADRICULAS.csv", sep=';')

    dataset['Time'] = pd.to_datetime(dataset['Datetime']).dt.time
    dataset["Date"] = pd.to_datetime(dataset["Datetime"], format='%Y-%m-%dT%H:%M:%S.%fZ').dt.date
    jogos_benfica["Data"] = pd.to_datetime(jogos_benfica["Data"], format='%Y-%m-%d').dt.date

    dataset["Game day"] = "No"
    for data in jogos_benfica["Data"]:
        dataset.loc[dataset['Date'] == data, "Game day"] = "Yes"

    grelha_lisboa.rename(columns={"grelha_id": "Grid_ID"}, inplace=True)
    dataset = pd.merge(dataset, grelha_lisboa[['Grid_ID', 'latitude', 'longitude']], on='Grid_ID', how='left')
    # --------------------------------------------------------------------------------------------

    # Criação colunas Datetime
    dataset['Datetime'] = pd.to_datetime(dataset['Datetime'])
    dataset['Year'] = dataset['Datetime'].dt.year
    dataset['Month'] = dataset['Datetime'].dt.month
    dataset['Day'] = dataset['Datetime'].dt.day
    dataset['Hour'] = dataset['Datetime'].dt.hour
    dataset['Minute'] = dataset['Datetime'].dt.minute

    # Filtração do Dataset para dias de jogo apenas e para horas de jogo
    dataset = dataset[dataset['Game day'] != 'No']
    dates_to_remove = ['2022-10-25', '2023-04-23']
    dataset = dataset[~dataset['Datetime'].dt.date.isin(pd.to_datetime(dates_to_remove).date)]
    dataset = dataset[(dataset['Hour'] >= 16) & (dataset['Hour'] <= 23)]
    dataset.reset_index(drop=True, inplace=True)

    dataset.to_csv("MobilidadeLisboa_previsão.csv")


def split_dataset(dataset):
    target_variable = 'C1'
    dataset['Year'] = pd.to_datetime(dataset['Datetime']).dt.year
    dataset['Month'] = pd.to_datetime(dataset['Datetime']).dt.month
    train_data = dataset[((dataset['Year'] == 2022) | ((dataset['Year'] == 2023) & (dataset['Month'] <= 2)))]
    test_data = dataset[(dataset['Year'] == 2023) & (dataset['Month'] > 2)]
    X_train = train_data.drop([target_variable, "Datetime"], axis=1)
    y_train = train_data[target_variable]
    X_test = test_data.drop([target_variable, "Datetime"], axis=1)
    y_test = test_data[target_variable]
    return X_train, y_train, X_test, y_test


def evaluation_models():
    dataset = pd.read_csv("MobilidadeLisboa_previsão.csv")

    target_variable = 'C1'
    categorical_columns = ['Game day']
    numerical_columns = dataset.columns.difference(categorical_columns + [target_variable, 'Datetime', 'Date', 'Time'])
    # Create transformers for numerical and categorical columns
    numerical_transformer = Pipeline(steps=[
        ('imputer', SimpleImputer(strategy='mean')),
        ('scaler', StandardScaler())
    ])
    categorical_transformer = Pipeline(steps=[
        ('imputer', SimpleImputer(strategy='most_frequent')),
        ('onehot', OneHotEncoder(handle_unknown='ignore'))
    ])
    # Create a preprocessor for both numerical and categorical columns
    preprocessor = ColumnTransformer(
        transformers=[
            ('num', numerical_transformer, numerical_columns),
            ('cat', categorical_transformer, categorical_columns)
        ])
    # Create a pipeline with the preprocessor and the linear regression model
    """model = Pipeline(steps=[
        ('preprocessor', preprocessor),
        ('regressor', LinearRegression())
    ])"""
    model = Pipeline(steps=[
        ('preprocessor', preprocessor),
        ('regressor', RandomForestRegressor(n_estimators=100, random_state=42))  # You can adjust hyperparameters
    ])
    """model = Pipeline(steps=[
        ('preprocessor', preprocessor),
        ('regressor', GradientBoostingRegressor(n_estimators=500, learning_rate=0.25, random_state=42))
    ])"""

    X_train, y_train, X_test, y_test = split_dataset(dataset)
    # Train the model
    model.fit(X_train, y_train)

    # Make predictions on the test set
    predictions = model.predict(X_test)

    # Evaluate the model
    mse = mean_squared_error(y_test, predictions)
    r2 = r2_score(y_test, predictions)

    print(f'Mean Squared Error: {mse}')
    print(f'R-squared: {r2}')

    dataset['Predicted_C1'] = model.predict(dataset.drop([target_variable, "Datetime"], axis=1))
    collumns_to_remove = ["Unnamed: 0", 'C3', 'Year', 'Month', 'Day', 'Hour', 'Minute']
    dataset = dataset.drop(collumns_to_remove, axis=1)
    dataset.to_csv("MobilidadeLisboa_previsão_limpo.csv")


def main():
    #dataset = pd.read_csv("MobilidadeLisboa.csv")
    #clean_dataset(dataset)
    evaluation_models()


if __name__ == "__main__":
    main()
